/*****************************************
  xjump version 2

  resource.h   フォールバックリソース

  (C) Nov 8, 1997  ROYALPANDA
*****************************************/

#include<stdio.h>


char *DefaultResources[] = {

"XJumpX*font                     : -adobe-*-*-*-*-*-24-*-*-*-*-*-*-*",
"XJumpX*foreground               : white",
"XJumpX*background               : black",
"XJumpX*borderColor              : white",
"XJumpX*defaultDistance          : 16",
"XJumpX*borderWidth              : 0",

"XJumpX*title.label              : FALLING TOWER",
"XJumpX*title.font               : -*-times-medium-r-*-*-33-*-*-*-*-*-*-*",
"XJumpX*title.horizDistance      : 64",
"XJumpX*title.foreground         : cyan",
"XJumpX*title.background         : black",
"XJumpX*title.borderWidth        : 0",

"XJumpX*title2.label              : c",
"XJumpX*title2.font               : -adobe-symbol-*-*-*-*-33-*-*-*-*-*-*-*",
"XJumpX*title2.foreground         : yellow",
"XJumpX*title2.background         : black",
"XJumpX*title2.fromHoriz          : title",
"XJumpX*title2.horizDistance      : 2",


"XJumpX*scoreBoard.fromVert      : title",
"XJumpX*scoreBoard.horizDistance : 128",

"XJumpX*scLabel.label            : Floor",

"XJumpX*score.fromHoriz          : scLabel",

"XJumpX*gameover.label           : Game Over",
"XJumpX*gameover.fromVert        : scoreBoard",
"XJumpX*gameover.horizDistance   : 200",
"XJumpX*gameover.vertDistance    : 96",
"XJumpX*gameover.foreground      : white",
"XJumpX*gameover.background      : blue",
"XJumpX*gameover.borderColor     : navy",
"XJumpX*gameover.borderWidth     : 4",

"XJumpX*pause.label              : Pause",
"XJumpX*pause.fromVert           : scoreBoard",
"XJumpX*pause.horizDistance      : 224",
"XJumpX*pause.vertDistance       : 96",
"XJumpX*pause.foreground         : white",
"XJumpX*pause.background         : blue",
"XJumpX*pause.borderColor        : navy",
"XJumpX*pause.borderWidth        : 4",

"XJumpX*scr.fromVert             : scoreBoard",
"XJumpX*scr.borderWidth          : 0",

"XJumpX*record_v.fromVert        : scoreBoard",
"XJumpX*record_v.allowVert       : True",
"XJumpX*record_v.allowHoriz      : True",
"XJumpX*record_v*borderWidth     : 1",

"XJumpX*record.font		: -*-fixed-*-*-*-*-20-*-*-*-c-*-*-*",

"XJumpX*copyright.label          : (C) 1999 ROYALPANDA",
"XJumpX*copyright.foreground     : green",
"XJumpX*copyright.fromVert       : scr",
"XJumpX*copyright.horizDistance  : 64",

"XJumpX*game.translations        : #override \\n\
\
<Key>Left    : key_on(LEFT)          \\n\
<KeyUp>Left  : key_off(LEFT)         \\n\
<Key>j       : key_on(LEFT)          \\n\
<KeyUp>j     : key_off(LEFT)         \\n\
<Key>KP_4    : key_on(LEFT)          \\n\
<KeyUp>KP_4  : key_off(LEFT)         \\n\
<Btn2Down>   : key_on(LEFT)          \\n\
<Btn2Up>     : key_off(LEFT)         \\n\
\
<Key>Right   : key_on(RIGHT)         \\n\
<KeyUp>Right : key_off(RIGHT)        \\n\
<Key>l       : key_on(RIGHT)         \\n\
<KeyUp>l     : key_off(RIGHT)        \\n\
<Key>KP_6    : key_on(RIGHT)         \\n\
<KeyUp>KP_6  : key_off(RIGHT)        \\n\
<Btn3Down>   : key_on(RIGHT)         \\n\
<Btn3Up>     : key_off(RIGHT)        \\n\
\
<Key>Up      : key_on(UP)start(0)     \\n\
<KeyUp>Up    : key_off(UP)           \\n\
<Key>space   : key_on(UP)start(0)     \\n\
<KeyUp>space : key_off(UP)           \\n\
<Key>z       : key_on(UP)            \\n\
<KeyUp>z     : key_off(UP)start(0)    \\n\
<Key>KP_8    : key_on(UP)            \\n\
<KeyUp>KP_8  : key_off(UP)start(0)    \\n\
<Btn1Down>   : key_on(UP)            \\n\
<Btn1Up>     : key_off(UP)start(0)    \\n\
<Key>s  : start(400)		\\n\
\
<Key>p       : pause()               \\n\
Shift<Key>q  : quit()			\\n\
",
NULL};
